//
// Created by lukas on 06.03.21.
//

#include "Clock.h"
#include "Types.h"

Clock::Clock() : strip(NUMPIXELS, D5, NEO_GRB + NEO_KHZ800), animator(), refreshTicker() {
}

void Clock::init() {
  strip.begin();
  strip.clear();
  strip.show();

  turnOn();
}

void Clock::turnOff() {
  strip.clear();
  strip.show();

  refreshTicker.detach();
}

void Clock::turnOn() {
  if (!refreshTicker.active())
    refreshTicker.attach(10, [this]() { this->refreshTime(); });
  this->refreshTime();
}

void Clock::paintAllwaysOnLeds() {
  printWord(types::es, Adafruit_NeoPixel::Color(150, 150, 0));
  printWord(types::ist, Adafruit_NeoPixel::Color(150, 0, 150));
}

void Clock::printWord(const std::vector<uint8_t>& word, uint32_t color) {
  for (const int i : word) {
    strip.setPixelColor(i, color);
  }
}

void Clock::refreshTime() {
  // grab the current instant in unix seconds
  time_t now = time(nullptr);
  if (now <= 604800) {
    animator.startLoadAnimation(&strip);
    return;
  }
  animator.stopLoadAnimation();
  strip.clear();

  tm* loctime = localtime(&now);

  const uint8_t hour = loctime->tm_hour;
  const uint8_t minute = loctime->tm_min;

  paintAllwaysOnLeds();
  setTime(hour, minute);
  strip.show();
  Serial.printf("Time now: %sh %sM", &hour, &minute);
}

void Clock::setTime(uint8_t hour, uint8_t minute) {
  const uint8_t minuteselector = minute / 5;

  // if minuteselector >= 4 +1 to hour
  if (minuteselector >= 4)
    hour++;

  // convert to 12h format
  if (hour >= 12)
    hour = hour - 12;

  std::vector<uint8_t> hourWord;
  hourWord = hour == 1                 ? (minuteselector == 0 ? types::ein : types::eins)  // eins only on a full hour
             : hour == 2               ? types::zwei
             : hour == 3               ? types::drei2
             : hour == 4               ? types::vier
             : hour == 5               ? types::fuenf2
             : hour == 6               ? types::sechs
             : hour == 7               ? types::sieben
             : hour == 8               ? types::acht
             : hour == 9               ? types::neun
             : hour == 10              ? types::zehn2
             : hour == 11              ? types::elf
             : hour == 0 || hour == 12 ? types::zwoelf
                                       : hourWord;

  printWord(hourWord, Adafruit_NeoPixel::Color(0, 150, 0));

  // fuenf / zehn / viertl word
  std::vector<uint8_t> minuteWord;
  if (minuteselector == 1 || minuteselector == 5 || minuteselector == 7 || minuteselector == 11)
    minuteWord = types::fuenf;
  else if (minuteselector == 2 || minuteselector == 4 || minuteselector == 8 || minuteselector == 10)
    minuteWord = types::zehn;
  else if (minuteselector == 3)
    minuteWord = types::viertel;
  else if (minuteselector == 9)
    minuteWord = types::dreiviertel;
  else if (minuteselector == 6)
    minuteWord = types::halb;

  printWord(minuteWord, Adafruit_NeoPixel::Color(0, 150, 0));

  // vor / nach
  std::vector<uint8_t> vornachWord;
  if (minuteselector == 1 || minuteselector == 2 || minuteselector == 3 || minuteselector == 7 || minuteselector == 8)
    vornachWord = types::nach;
  else if (minuteselector == 4 || minuteselector == 5 || minuteselector == 10 || minuteselector == 11)
    vornachWord = types::vor;
  printWord(vornachWord, Adafruit_NeoPixel::Color(150, 150, 150));

  // halb
  if (minuteselector >= 4 && minuteselector <= 8)
    printWord(types::halb, Adafruit_NeoPixel::Color(150, 0, 0));

  // uhr
  if (minuteselector == 0)
    printWord(types::uhr, Adafruit_NeoPixel::Color(0, 0, 150));
}